@@ -11,17 +11,15 @@ fi
 echo "Releasing PIDA! Spread the love."
 
 # Tag the thing
-REPOURI="svn+ssh://aafshar@svn.berlios.de/svnroot/repos/pida"
-SRCURI="$REPOURI/trunk"
-DSTURI="$REPOURI/tags/release-$VERSION"
-TAGMESSAGE="Tagged release $VERSION"
-svn cp -m"$TAGMESSAGE" $SRCURI $DSTURI
+#REPOURI="svn+ssh://aafshar@svn.berlios.de/svnroot/repos/pida"
+#SRCURI="$REPOURI/trunk"
+#DSTURI="$REPOURI/tags/release-$VERSION"
+#TAGMESSAGE="Tagged release $VERSION"
+#svn cp -m"$TAGMESSAGE" $SRCURI $DSTURI
 
 # Make the dist tarball
-rm setup.cfg
-rm data/version
+rm pida/data/version
 python setup.py -q sdist
-svn revert setup.cfg
 SDIST_TARBALL="dist/pida-$VERSION.tar.gz"
 
 
@@ -30,4 +28,7 @@ BERLIOS_URL="ftp.berlios.de"
 BERLIOS_DIR="incoming"
 ncftpput $BERLIOS_URL $BERLIOS_DIR $SDIST_TARBALL
 
+#upload to pypy
+#python setup.py bdist_egg sdist upload
+
 
