@@ -26,8 +26,7 @@ import gtk
 import gobject
 
 # pidagtk import(s)
-import toolbar
-from pida.utils.kiwiutils import gsignal, gproperty
+from pida.utils.kiwiutils import gproperty
 
 
 class TreeItem(object):
@@ -58,13 +57,19 @@ class QuestionBox(gtk.HBox):
         gtk.HBox.__init__(self)
         self.__entry = gtk.Entry()
         self.pack_start(self.__entry)
-        self.__toolbar = toolbar.Toolbar()
-        self.pack_start(self.__toolbar)
-        self.__toolbar.add_button('ok', 'apply', 'ok')
-        self.__toolbar.add_button('cancel', 'cancel', 'cancel')
-        self.__toolbar.connect('clicked', self.cb_toolbar_clicked)
+        self.pack_start(self.__init_toolbar())
         self.set_sensitive(False)
 
+    def __init_toolbar(self):
+        hb = gtk.HBox()
+        b2 = gtk.ToolButton(stock_id=gtk.STOCK_CANCEL)
+        hb.pack_start(b2)
+        b2.connect('clicked', self.cb_toolbar_clicked, 'cancel')
+        b1 = gtk.ToolButton(stock_id=gtk.STOCK_YES)
+        hb.pack_start(b1)
+        b1.connect('clicked', self.cb_toolbar_clicked, 'ok')
+        return hb
+        
     def cb_toolbar_clicked(self, toolbar, action):
         if action == 'ok':
             self.__current_callback(self.__entry.get_text())
@@ -269,9 +274,14 @@ class Tree(gtk.VBox):
         row = [key, titem, self.__get_markup(item)]
         niter = self.model.append(parent, row)
         def reset():
-            self.model.set_value(niter, 2, self.get_markup(item))
-        titem.reset_markup = reset
-        item.reset_markup = reset
+            for model, ite in item._tree_models:
+                model.set_value(ite, 2, self.get_markup(item))
+        for obj in [item, titem]:
+            if not hasattr(obj, 'reset_markup'):
+                obj.reset_markup = reset
+            if not hasattr(obj, '_tree_models'):
+                obj._tree_models = []
+            obj._tree_models.append([self.model, niter])
         return niter
 
     def __get_markup(self, item):
@@ -280,6 +290,7 @@ class Tree(gtk.VBox):
         for k in dir(item):
             if k in markup_fmt:
                 unmangled[k] = getattr(item, k)
+
         markup_string = markup_fmt % unmangled
         return markup_string
     get_markup = __get_markup
@@ -336,6 +347,9 @@ class Tree(gtk.VBox):
         self.__model.set_sort_func(sortcolid, comparemethod)
         #self.__view.set_model(self.__model)
 
+    def grab_focus(self):
+        self.__view.grab_focus()
+
     def __get_model(self):
         """Return the Tree Model."""
         return self.__model
@@ -409,7 +423,8 @@ class Tree(gtk.VBox):
             direction = -1#gtk.SORT_ASCENDING
         self.sort_by([self.sort_available[text]], direction=direction)
 
-gobject.type_register(Tree)
+if gtk.gtk_version < (2, 6):
+    gobject.type_register(Tree)
 
 class IconTreeItem(TreeItem):
     """I tree item with an icon."""
@@ -452,9 +467,15 @@ class IconTree(Tree):
         row = [key, titem, self.get_markup(item), pixbuf]
         niter = self.model.append(parent, row)
         def reset():
-            self.model.set_value(niter, 2, self.get_markup(item))
-        titem.reset_markup = reset
-        item.reset_markup = reset
+            for model, ite in item._tree_models:
+                model.set_value(ite, 2, self.get_markup(item))
+                model.set_value(ite, 3, item.pixbuf)
+        for obj in [item, titem]:
+            if not hasattr(obj, 'reset_markup'):
+                obj.reset_markup = reset
+            if not hasattr(obj, '_tree_models'):
+                obj._tree_models = []
+            obj._tree_models.append([self.model, niter])
         return niter
 
 class ToggleTree(Tree):
