@@ -23,27 +23,23 @@
 
 
 import pida.core.service as service
-import pida.core.registry as registry
-import pida.pidagtk.contentbook as contentbook
-import pida.pidagtk.contentview as contentview
-import pida.pidagtk.filedialogs as filedialogs
-import pida.pidagtk.tree as tree
+from  pida.pidagtk import contentview, filedialogs, tree
 
 #import pida.utils.pygrep as pygrep
-
 import os
 import re
-import time
 import gobject
 import threading
 import linecache
-import cgi
 import string
 import gtk
+from gtk import gdk
 
-import pida.core.actions as actions
+from pida.core import actions
+from pida.utils.gthreads import locked, gcall
+from gobject import markup_escape_text
 
-types = service.types
+from pida.model import attrtypes as types
 defs = service.definitions
 
 table = range(0,128) * 2
@@ -70,8 +66,8 @@ SMALL_MU = ('<span size="small" weight="bold">%s</span>')
 class ResultsTreeItem(tree.TreeItem):
     
     def get_markup(self):
-        return RESULT_MU % (self.value.linenumber,
-                            cgi.escape(self.value.filename),
+        return RESULT_MU % (self.value.linenumber + 1,
+                            markup_escape_text(self.value.filename),
                             self.value.muline)
     markup = property(get_markup)
 
@@ -159,7 +155,7 @@ class GrepView(contentview.content_view):
     def cb_result_selected(self, tree, result):
         #lines = self.boss.option('grepper', 'context-lines')
         lines = 4
-        pre, match, post = [cgi.escape(s) for s in
+        pre, match, post = [markup_escape_text(s) for s in
                             result.value.get_context(lines)]
         self.__context_label.set_markup(RESULT_CONTEXT_MU % (pre, match, post))
 
@@ -213,34 +209,50 @@ class GrepView(contentview.content_view):
     def cb_activated(self, entry):
         self.service.grep_start()
 
-class Grepper(service.service):
-
-    single_view_type = GrepView
-    single_view_book = 'view'
-
-    display_name = 'Grep Search'
-
+class GrepConfig:
+    __order__  = ['default_options', 'results']
     class default_options(defs.optiongroup):
         """Options that the search will start with by default."""
+        __order__ = ['start_detailed', 'recursive_search',
+                     'ignore_version_control_directories']
+        label = 'Default Search Options'
         class start_detailed(defs.option):
             """Whether the detailed search options will start expanded."""
+            label = 'Start with details visible'
             rtype = types.boolean
             default = False
         class recursive_search(defs.option):
             """Whether the search will be recursive by default."""
             rtype = types.boolean
             default = True
+            label = 'Recursive search'
         class ignore_version_control_directories(defs.option):
             """Whether version control directories will be ignored by default."""
             rtype = types.boolean
             default = True
+            label = 'Ignore version control directories'
     class results(defs.optiongroup):
         """Options relating to search results."""
+        __order__ = ['maximum_results']
+        label = 'Result Options'
         class maximum_results(defs.option):
             """The maximum number of search results."""
+            label = 'Maximum number of results'
             rtype = types.intrange(5, 5000, 5)
             default = 500
 
+    __markup__ = lambda self: 'Text Searcher'
+
+class Grepper(service.service):
+
+    config_definition = GrepConfig
+
+    class GrepView(defs.View):
+        view_type = GrepView
+        book_name = 'view'
+
+    display_name = 'Grep Search'
+
     def grep_start(self):
         opts = self.single_view.get_options()
         if not opts.pattern:
@@ -261,47 +273,48 @@ class Grepper(service.service):
 
     def cb_results_status(self, grep, status):
         self.single_view.show_status(status)
-            
 
     def cmd_find_interactive(self, directories=None, ignorevcs=None,
                              recursive=None):
-        self.create_single_view()
+        
+        opts = self.options.default_options
+        view = self.create_view('GrepView')
         options = GrepOptions()
+
+        self.show_view(view=view)
+
         if directories is None:
             proj = self.boss.call_command('projectmanager',
                                           'get_current_project')
+
             if proj is not None:
-                options.directories = [proj.source_directory]
+                options.directories = [proj.source__directory]
             else:
                 options.directories = [os.getcwd()]
         else:
             options.directories = directories
         if ignorevcs is None:
-            options.ignorevcs = self.opt(
-                'default_options', 'ignore_version_control_directories')
+            options.ignorevcs = opts.ignore_version_control_directories
         else:
             options.ignorevcs = ignorevcs
         if recursive is None:
-            options.recursive = self.opt(
-                'default_options', 'recursive_search')
+            options.recursive = opts.recursive_search
         else:
             options.recursive = recursive
-        options.maxresults = self.opt('results', 'maximum_results')
+        options.maxresults = self.opts.results__maximum_results
         self.single_view.from_options(options)
-        self.single_view.set_details_expanded(self.opt(
-           'default_options', 'start_detailed'))
+        self.single_view.set_details_expanded(opts.start_detailed)
 
     def cmd_find(self, path, pattern):
         pass
 
-
     def cb_search_clicked(self, button):
         self.cmd_find_interactive()
 
     def cb_view_action(self, view, name):
         if name == 'apply':
             self.grep()
-        if name == 'stop':
+        elif name == 'stop':
             self.__grep.stop()
 
     @actions.action(stock_id='gtk-searchtool',
@@ -334,10 +347,13 @@ class Grepper(service.service):
                 <placeholder name="ToolsToolbar">
             <separator />
             <toolitem  name="grepper" action="grepper+find" />
-            <separator />
                 </placeholder>
                 </toolbar>
             """
+
+    def get_single_view(self):
+        return self.get_first_view('GrepView')
+    single_view = property(get_single_view)
     
     
 BINARY_RE = re.compile(r'[\000-\010\013\014\016-\037\200-\377]')
@@ -360,9 +376,9 @@ class GrepResult(object):
         muline = ''
         for match in matches:
             prematch, line = line.split(match, 1)
-            muline = '%s%s' % (muline, cgi.escape(prematch))
-            muline = '%s%s' % (muline, MATCH_MU % cgi.escape(match))
-        self.muline = '%s%s' % (muline, cgi.escape(line))
+            muline = '%s%s' % (muline, markup_escape_text(prematch))
+            muline = '%s%s' % (muline, MATCH_MU % markup_escape_text(match))
+        self.muline = '%s%s' % (muline, markup_escape_text(line))
 
     def get_context(self, lines=2):
         pre = []
@@ -379,12 +395,15 @@ class GrepResult(object):
 
     def get_markup(self):
         return RESULT_MU % (self.linenumber,
-                            cgi.escape(self.filename),
+                            markup_escape_text(self.filename),
                             self.muline)
     markup = property(get_markup)
 
 class PidaGrep(gobject.GObject):
-    
+    """An object that contains two signals, 'found' and 'status',
+    used to monitor its state. This object greps through a group of
+    files."""
+
     __gsignals__ = {'found' : (
                         gobject.SIGNAL_RUN_LAST,
                         gobject.TYPE_NONE,
@@ -400,74 +419,96 @@ class PidaGrep(gobject.GObject):
         self.__pattern = re.compile(options.pattern)
         self.__t = None
         self.__running = False
+        self.lock = threading.Lock()
 
+    running = property(lambda x: x.__running,
+                       doc="Returns if this operation is running")
+
+    @locked("lock")
     def run(self):
-        self.__t = threading.Thread(target = self.__run)
-        self.__t.start()
+        if self.running:
+            return
         self.__running = True
+        threading.Thread(target=self._run).start()
 
-    def get_files(self):
+    def _get_files(self):
         for directory in self.__options.directories:
             for dirname, dirnames, filenames in os.walk(directory):
                 if not self.__options.recursive:
                     dirnames[0:] = []
                 else:
                     if self.__options.ignorevcs:
-                        for d in dirnames:
-                            if d in ['.svn', 'CVS', '_darcs']:
-                                dirnames.remove(d)
+                        for vcs_dir in ['.svn', 'CVS', '_darcs', '.bzr']:
+                            try:
+                                dirnames.remove(vcs_dir)
+                            except ValueError:
+                                pass
+
                 for filename in filenames:
                     filepath = os.path.join(dirname, filename)
                     yield filepath
         for filepath in self.__options.files:
                 yield filepath
 
-    def __run(self):
-        try:
-            self.__find()
-        except StopIteration:
-            return
-
-    def __find(self):
-        self.status(1, "searching")
-        self.__nfound = 0
-        for i, filename in enumerate(self.get_files()):
+    def _find_files(self):
+        self._status(1, "searching")
+        found = 0
+        
+        for i, filename in enumerate(self._get_files()):
+            if not self.running or found >= self.__options.maxresults:
+                break
+            
+            # update status from time to time
             if i % 16 == 0:
-                self.status(self.__nfound)
+                self._status(found)
+                
             try:
                 f = open(filename, 'r')
             except IOError:
                 continue
+
             for linenumber, line in enumerate(f):
-                if not self.__running:
-                    self.__finished()
+                if not self.running:
+                    break
+                
+                # abort binary files
                 if BINARY_RE.match(line):
                     break
+                    
                 line = string.translate(line, all_chars, hi_bit_chars)
                 line = string.translate(line, hi_lo_table)
                 matches = self.__pattern.findall(line)
+                
                 if len(matches):
-                    self.__nfound = self.__nfound + len(matches)
-                    if self.__nfound >= self.__options.maxresults:
-                        self.__finished()
+                    found = found + len(matches)
+                    
+                    # break when we have found enough values
+                    if found >= self.__options.maxresults:
+                        break
+                    
+                    # emit the result we've found
                     result = GrepResult(linenumber, filename, line, matches)
-                    gtk.threads_enter()
-                    self.emit('found', result)
-                    gtk.threads_leave()
+                    gcall(self.emit, 'found', result)
+                    
             f.close()
-        self.__finished()
+            
+                
+        return found
 
-    def status(self, code, message=None):
-        gtk.threads_enter()
-        self.emit('status', (code, message))
-        gtk.threads_leave()
+    def _run(self):
+        self._status(1, '%s found' % self._find_files())
 
-    def stop(self):
+        self.lock.acquire()
         self.__running = False
+        self.lock.release()
+        
+    def _status(self, code, message=None):
+        gcall(self.emit, 'status', (code, message))
 
-    def __finished(self):
-        self.status(1, '%s found' % self.__nfound)
-        raise StopIteration
+    @locked("lock")
+    def stop(self):
+        """Use this method to stop this operation"""
+        self.__running = False
 
 gobject.type_register(PidaGrep)
 
