@@ -21,22 +21,17 @@
 #OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 #SOFTWARE.
 
-import pida.pidagtk.tree as tree
-import pida.pidagtk.configview as configview
-import pida.pidagtk.contentbook as contentbook
-import pida.pidagtk.contentview as contentview
-import pida.pidagtk.contextwidgets as contextwidgets
+import os
+
+import gobject, gtk
+
+from pida.pidagtk import tree, contentview
+from pida.core import service, actions, project
+from pida.model import model, views, persistency
 
-import pida.core.registry as registry
-import pida.core.service as service
-import pida.core.actions as actions
-types = service.types
 defs = service.definitions
 
-import gtk
-import os
-import gobject
-import os.path
+
 
 class Project(object):
 
@@ -46,6 +41,8 @@ class Project(object):
     vcs = None
     vcsname = None
 
+
+
 class ProjectTreeItem(tree.TreeItem):
 
     def __get_markup(self):
@@ -59,7 +56,9 @@ class ProjectTreeItem(tree.TreeItem):
         return b
     markup = property(__get_markup)
 
-class ProjectTree(tree.Tree):
+
+
+class ProjectTree(views.TreeObserver):
 
     SORT_BY = 'name'
 
@@ -76,6 +75,7 @@ class ProjectTree(tree.Tree):
         self.view.set_expander_column(self.view.get_column(1))
 
 
+
 class project_view(contentview.content_view):
     SHORT_TITLE = 'Projects'
     LONG_TITLE = 'List of projects'
@@ -87,56 +87,42 @@ class project_view(contentview.content_view):
     HAS_TITLE = False
 
     def init(self):
-        self.__toolbar = contextwidgets.context_toolbar()
-        self.widget.pack_start(self.__toolbar, expand=False)
-        self.__projectlist = ProjectTree()
-        self.widget.pack_start(self.__projectlist)
-        self.__projectlist.set_property('markup-format-string',
-            self.__projectlist.markup_format_string)
-        self.__projectlist.connect('clicked',
+        self.proj_list = self.service.proj_group.create_multi_observer(
+                                                    ProjectTree)
+        self.proj_list.connect('clicked',
             self.service.cb_plugin_view_project_clicked)
-        self.__projectlist.connect('double-clicked',
+        self.proj_list.connect('double-clicked',
             self.service.cb_plugin_view_project_double_clicked)
-        self.__projectlist.connect('right-clicked',
+        self.proj_list.connect('right-clicked',
             self.cb_list_right_clicked)
-
-    def set_projects(self, *args):
-        self.__projectlist.set_projects(self.service.projects)
+        self.widget.pack_start(self.proj_list)
 
     def get_selected(self):
-        return self.__projectlist.selected
-
-    def set_contexts(self, contexts):
-        self.__toolbar.set_contexts(contexts)
+        return self.proj_list.selected
 
     def cb_list_right_clicked(self, treeview, item, event):
-        menu = gtk.Menu()
         if item is None:
+            menu = gtk.Menu()
             for act in ['projectmanager+new_project',
                         'projectmanager+add_project']:
                 action = self.service.action_group.get_action(act)
                 mi = action.create_menu_item()
                 menu.add(mi)
         else:
-            globaldict = {'directory': item.value.source_directory,
-                          'project': item.value}
-            for title, context in [('Directory', 'directory'),
-                            ('Source code', 'project_directory'),
-                            ('Project', 'project')]:
-                mroot = gtk.MenuItem(label=title)
-                menu.add(mroot)
-                contexts = self.service.boss.call_command('contexts',
-                                         'get_contexts',
-                                         contextname=context,
-                                         globaldict=globaldict
-                                         )
-                cmenu = contextwidgets.get_menu(contexts)
-                mroot.set_submenu(cmenu)
+            menu = self.service.boss.call_command('contexts',
+                'get_context_menu', ctxname='directory',
+                ctxargs=[item.value.source__directory])
+            menu.add(gtk.SeparatorMenuItem())
+            for act in ['projectmanager+properties',
+                        'projectmanager+remove_project_from_workbench']:
+                action = self.service.action_group.get_action(act)
+                mi = action.create_menu_item()
+                menu.add(mi)
         menu.show_all()
         menu.popup(None, None, None, event.button, event.time)
 
     def get_model(self):
-        return self.__projectlist.model
+        return self.proj_list.model
 
     def set_selected(self, key):
         self.__projectlist.set_selected(key)
@@ -144,27 +130,36 @@ class project_view(contentview.content_view):
     def get_selected_iter(self):
         return self.__projectlist.selected_iter
 
+
+
+class ProjectEditor(contentview.content_view):
+
+    def init(self):
+        mg = self.service.proj_group
+        self.tv = mg.create_multi_observer(views.TreeObserver)
+        self.pp = mg.create_single_observer(views.PropertyPage)
+        b = gtk.HPaned()
+        b.pack1(self.tv)
+        b.pack2(self.pp)
+        b.set_position(200)
+        self.widget.pack_start(b)
+
+
+
 class ProjectManager(service.service):
     """Project Management"""
 
     # view definitions
 
-    plugin_view_type = project_view
+    class ProjectView(defs.View):
+        view_type = project_view
+        book_name = 'content'
 
-    single_view_type = configview.config_view
-    single_view_book = 'ext'
+    class ProjectEditor(defs.View):
+        view_type = ProjectEditor
+        book_name = 'ext'
 
-    class default(defs.project_type):
-        project_type_name = 'default'
-        class general(defs.optiongroup):
-            """General options for Python projects"""
-            class source_directory(defs.option):
-                """The directory containing source code."""
-                rtype = types.directory
-                default = os.path.expanduser('~')
-   
     # life cycle
- 
     def init(self):
         self.__current_project = None
         self.__history = []
@@ -172,10 +167,28 @@ class ProjectManager(service.service):
             'projects', 'projectlist.conf')
         self.__last_file = os.path.join(self.boss.pida_home,
             'projects', 'projectlast.conf')
+        self.__started = False
+        self.__init_model()
         if not os.path.exists(self.__history_file):
             self.__write_history()
-        self.__started = False
+        self.__read_history()
+        self.create_view('ProjectView')
         self.__init_project_toolbar()
+        self._editview = None
+
+    def __init_model(self):
+        def _act(item):
+            self.__current_project_activated()
+        self.proj_group = model.ModelGroup(_act)
+        self.__projects_combo = self.proj_group.create_multi_observer(
+            views.ComboObserver)
+        self.ini_watch = self.proj_group.create_single_observer(
+            persistency.IniFileObserver)
+        self.act_watch = self.proj_group.create_single_observer(
+            views.ActionSensitivityObserver)
+        act = self.action_group.get_action('projectmanager+execute_project')
+        act.set_sensitive(False)
+        self.act_watch.add_widget(act, 'execution__uses')
 
     def __init_project_toolbar(self):
         tb = self.boss.call_command('window', 'get_ui_widget',
@@ -185,114 +198,86 @@ class ProjectManager(service.service):
         i = tb.get_item_index(ph)
         ti = gtk.ToolItem()
         tb.insert(ti, i)
-        #ti.set_expand(True)
         tbox = gtk.HBox(spacing=6)
         ti.add(tbox)
-        l = gtk.Label('Project')
-        tbox.pack_start(l, expand=False)
-        self.__projects_combo = gtk.ComboBox()
         tbox.pack_start(self.__projects_combo)
-        cell = gtk.CellRendererText()
-        self.__projects_combo.pack_start(cell, True)
-        self.__projects_combo.set_attributes(cell, text=0)
-        self.__cmb_con = self.__projects_combo.connect_after('changed',
-                         self.cb_combo_changed)
-        tb.insert(gtk.SeparatorToolItem(), i)
-
 
     def reset(self):
-        if not self.__started:
-            self.__started = True
-            if os.path.exists(self.__last_file):
-                f = open(self.__last_file, 'r')
-                name = f.read()
-                f.close()
-                self.plugin_view.set_selected(name)
-                self.__current_project_activated()
+        self.__started = True
+        if os.path.exists(self.__last_file):
+            f = open(self.__last_file, 'r')
+            name = f.read().strip()
+            f.close()
+            for proj in self.proj_group:
+                if proj.general__name == name:
+                    self.proj_group.set_current(proj)
+                    self.__current_project_changed(proj)
+                    self.__current_project_activated()
 
     def stop(self):
         if self.__current_project is not None:
             f = open(self.__last_file, 'w')
-            f.write(self.__current_project.name)
+            f.write(self.__current_project.general__name)
             f.close()
 
-    def bnd_filemanager_directory_changed(self, directory):
-        if self.__current_project is not None:
-            self.__current_project.browse_directory = directory
-
     # private interface
 
     def __set_toolbar(self):
         self.__projects_combo.set_model(self.plugin_view.get_model())
+        self.__projects_combo.set_property('text-column', 2)
+        entry = self.__projects_combo.get_children()[0]
+        entry.set_editable(False)
 
     def __read_history(self):
-        self.__history = []
         f = open(self.__history_file, 'r')
+        hist = set()
         for filename in f:
-            self.__history.append(filename.strip())
+            filename = filename.strip()
+            if os.path.exists(filename):
+                hist.add(filename)
         f.close()
+        for filename in hist:
+            p = project.Project()
+            persistency.load_model_from_ini(filename, p)
+            p.__model_ini_filename__ = filename
+            self.proj_group.add_model(p)
+        self.__write_history()
 
     def __write_history(self):
         f = open(self.__history_file, 'w')
-        for filename in self.__history:
-            f.write('%s\n' % filename)
+        for proj in set(self.proj_group):
+            f.write('%s\n' % proj.general__filename)
         f.close()
 
-    def __update(self):
-        self.__read_history()
-        self.__load_projects()
-        self.plugin_view.set_projects()
-        self.__set_toolbar()
-
-    def __load_projects(self):
-        self.__projects = []
-        for filename in self.__history:
-            project = self.boss.call_command('projecttypes',
-                        'load_project', project_file_name=filename)
-            if project is not None:
-                self.__projects.append(project)
-             
-
     def __launch_editor(self, projects=None, current_project=None):
-        if projects is None:
-            projects = self.projects
-        view = self.create_single_view()
-        view.connect('data-changed', self.cb_data_changed)
-        view.set_registries([(p.name, p.options) for p in projects])
-        if current_project is not None:
-            self.single_view.show_page(current_project.name)
+        if self._editview is None:
+            self._editview = self.create_view('ProjectEditor')
+            self.show_view(view=self._editview)
+        self._editview.raise_page()
 
     def __current_project_changed(self, project):
         if project is not self.__current_project:
-            if self.__current_project is not None:
-                self.__current_project.project_type.action_group.\
-                    set_visible(False)
             self.__current_project = project
-            self.__current_project.project_type.action_group.set_visible(True)
-            self.boss.call_command('window', 'update_action_groups')
-            ite = self.plugin_view.get_selected_iter()
-            self.__projects_combo.disconnect(self.__cmb_con)
-            self.__projects_combo.set_active_iter(ite)
-            self.__cmb_con = self.__projects_combo.connect_after('changed',
-                         self.cb_combo_changed)
 
     def __current_project_activated(self):
-        directory = self.__current_project.browse_directory
-        if directory is not None:
-            self.boss.call_command('filemanager', 'browse',
-                                    directory=directory)
+        if self.__current_project is not None:
+            directory = self.__current_project.general__browse_directory
+            if directory is not None:
+                self.boss.call_command('filemanager', 'browse',
+                                        directory=directory)
 
     # external interface
-   
     def get_projects(self):
-        for project in self.__projects:
+        for project in self.proj_group:
             yield project
     projects = property(get_projects)
 
-    # bindings
-    
-    def bnd_projecttypes_project_type_registered(self):
-        self.__update()
+    # bidings
+    def bnd_filemanager_directory_changed(self, directory):
+        if self.__current_project is not None:
+            def set_directory(directory):
+                self.__current_project.general__browse_directory = directory
+            gobject.idle_add(set_directory, directory)
 
     #commands
 
@@ -301,16 +286,52 @@ class ProjectManager(service.service):
             current_project = self.__current_project
         self.__launch_editor(projects, current_project)
 
+    def cmd_create_project(self):
+        chooser = gtk.FileChooserDialog("",
+                        self.boss.get_main_window(),
+                        gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
+                        (
+                        gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
+                        "Create Project", gtk.RESPONSE_ACCEPT,)
+                        )
+        chooser.set_action(gtk.FILE_CHOOSER_ACTION_SAVE)
+        filt = gtk.FileFilter()
+        filt.add_pattern('*.pida')
+        chooser.set_filter(filt)
+        try:
+            chooser.set_do_overwrite_confirmation(True)
+        except AttributeError:
+            pass
+        def response(dialog, resp):
+            chooser.hide()
+            fn = dialog.get_filename()
+            chooser.destroy()
+            if resp != gtk.RESPONSE_ACCEPT:
+                return
+            p = project.Project()
+            if not fn.endswith('.pida'):
+                fn = '%s.pida' % fn
+            p.__model_ini_filename__ = fn
+            self.proj_group.add_model(p)
+            p.general__name = os.path.basename(fn).split('.')[0]
+            self.proj_group.set_current(p)
+            self.__write_history()
+            self.cmd_edit()
+        chooser.connect('response', response)
+        chooser.show_all()
+
+
     def cmd_add_project(self, project_file):
-        self.__history.append(project_file)
-        self.__history = list(set(self.__history))
+        p = project.Project()
+        m = persistency.load_model_from_ini(project_file, p)
+        m.__model_ini_filename__ = project_file
+        self.proj_group.add_model(m)
         self.__write_history()
-        self.__update()
+        #self.__update()
 
     def cmd_remove_project(self, project):
-        self.__history.remove(project.project_filename)
+        self.proj_group.remove_model(project)
         self.__write_history()
-        self.__update()
 
     def cmd_get_current_project(self):
         return self.__current_project
@@ -319,35 +340,42 @@ class ProjectManager(service.service):
         return self.projects
 
     def cmd_get_project_for_file(self, filename):
+        if filename is None:
+            return None
         for project in self.projects:
-            if project.source_directory in filename:
+            if project.source__directory in filename:
                 return project
         return None
-        
 
     # Actions
 
     @actions.action(stock_id='gtk-new')
     def act_new_project(self, action):
         """Create a new project."""
-        self.boss.call_command('projectcreator', 'create_interactive')
+        self.cmd_create_project()
 
     def act_properties(self, action):
         """See or edit the properties for this project."""
         self.call('edit')
 
+    @actions.action(
+        stock_id=gtk.STOCK_ADD,
+        label="Add Existing Project...",
+    )
     def act_add_project(self, action):
-        fdialog = gtk.FileChooserDialog('Please select the project file',
+        fdialog = gtk.FileChooserDialog('',
                                  parent=self.boss.get_main_window(),
                                  action=gtk.FILE_CHOOSER_ACTION_OPEN,
-                                 buttons=(gtk.STOCK_OK,
-                                          gtk.RESPONSE_ACCEPT,
-                                          gtk.STOCK_CANCEL,
-                                          gtk.RESPONSE_REJECT))
+                                 buttons=(gtk.STOCK_CANCEL,
+                                          gtk.RESPONSE_REJECT,
+                                          "Add Project",
+                                          gtk.RESPONSE_ACCEPT,))
+        
         def response(dialog, response):
             if response == gtk.RESPONSE_ACCEPT:
                 self.call('add_project', project_file=dialog.get_filename())
             dialog.destroy()
+
         fdialog.connect('response', response)
         fdialog.run()
 
@@ -356,22 +384,32 @@ class ProjectManager(service.service):
     def act_commit_project(self, action):
         """Commit the current project to version control"""
         if self.__current_project is not None:
-            directory = self.__current_project.source_directory
+            directory = self.__current_project.source__directory
             self.boss.call_command('versioncontrol', 'commit',
                                    directory=directory)
 
-    #def act_get_project_statuses(self, action):
-    #    pass
-
     @actions.action(stock_id='vcs_update',
                     default_accel='<Shift><Control>u')
     def act_update_project(self, action):
         """Update the current project from version control"""
         if self.__current_project is not None:
-            directory = self.__current_project.source_directory
+            directory = self.__current_project.source__directory
             self.boss.call_command('versioncontrol', 'update',
                                 directory=directory)
 
+    @actions.action(stock_id='gtk-project',
+                    default_accel='<Shift><Control>x')
+    def act_execute_project(self, action):
+        if self.__current_project is None: return
+        com = self.__current_project.execution__command
+        """Execute the current project."""
+        shell_cmd = 'sh'
+        if com:
+            self.boss.call_command('terminal', 'execute',
+                command_args=[shell_cmd, '-c', com], icon_name='run')
+        else:
+            self.log.info('project has not set an executable')
+
     @actions.action(stock_id='gtk-remove')
     def act_remove_project_from_workbench(self, action):
         self.call('remove_project', project=self.__current_project)
@@ -403,9 +441,10 @@ class ProjectManager(service.service):
             self.databases['project_data'].sync()
             self.cb_data_changed('project_data')
 
-    def cb_data_changed(self, configview):
-        self.__update()
-
+    def view_closed(self, view):
+        self.proj_group.remove_observer(view.pp)
+        self.proj_group.remove_observer(view.tv)
+        self._editview = None
     # ui definition
 
     def get_menu_definition(self):
@@ -416,6 +455,7 @@ class ProjectManager(service.service):
                 <menu name="base_edit" action="base_edit_menu">
                 </menu>
                 <menu name="base_project" action="base_project_menu">
+                <placeholder name="ProjectMain">
                 <menuitem name="newproj" action="projectmanager+new_project" />
                 <menuitem name="addproj" action="projectmanager+add_project" />
                 <separator />
@@ -426,6 +466,7 @@ class ProjectManager(service.service):
                 <menuitem name="upproj" action="projectmanager+update_project" />
                 <menuitem name="comproj" action="projectmanager+commit_project" />
                 <separator />
+                </placeholder>
                 </menu>
                 <menu name="base_tools" action="base_tools_menu">
                 </menu>
@@ -438,9 +479,9 @@ class ProjectManager(service.service):
             <placeholder name="EditToolbar">
             </placeholder>
             <placeholder name="ProjectToolbar">
+                <toolitem name="exproj" action="projectmanager+execute_project" />
             </placeholder>
             <placeholder name="VcToolbar">
-                <separator />
                 <toolitem name="upproj" action="projectmanager+update_project" />
                 <toolitem name="comproj" action="projectmanager+commit_project" />
                 <separator />
@@ -448,6 +489,10 @@ class ProjectManager(service.service):
             </toolbar>
         """
 
+    def get_plugin_view(self):
+        return self.get_first_view('ProjectView')
+    plugin_view = property(get_plugin_view)
+
 
 Service = ProjectManager
 
