@@ -23,92 +23,107 @@
 
 import os
 
-import gtk
+import gtk, gobject
 
-# XXX: should be importing from rat
-#from pida.utils import shiftpaned
-from pida.pidagtk import shiftpaned
+from rat import shiftpaned
 
 import pida.core.actions as actions
 import pida.core.service as service
-import pida.pidagtk.window as window
-import pida.pidagtk.expander as expander
-import pida.pidagtk.contentbook as contentbook
+
+from pida.pidagtk.contentview import ContentManager, create_pida_icon
 
 
-types = service.types
+from pida.model import attrtypes as types
 defs = service.definitions
 
-from pkg_resources import Requirement, resource_filename
-icon_file = resource_filename(Requirement.parse('pida'),
-                              'pida-icon.png')
-im = gtk.Image()
-im.set_from_file(icon_file)
-im2 = gtk.Image()
-im2.set_from_file(icon_file)
 
-class SplashWindow(gtk.Window):
+class WindowConfig:
 
-    def __init__(self):
-        gtk.Window.__init__(self, gtk.WINDOW_POPUP)
-        self.set_position(gtk.WIN_POS_CENTER)
-        hb = gtk.HBox()
-        hb.set_property('border-width', 36)
-        self.add(hb)
-        hb.pack_start(im2, padding=12)
-        self._msg = gtk.Label()
-        self._msg.set_alignment(0.8, 0.8)
-        hb.pack_start(self._msg)
-        
-    def message(self, msg):
-        self._msg.set_markup(msg)
+    __order__ = ['layout', 'window_size', 'toolbar_and_menubar']
+    class layout(defs.optiongroup):
+        """The layout options."""
+        __order__ = ['sidebar_on_right', 'vertical_sidebar_split',
+                      'small_toolbar', 'sidebar_width']
+        label = 'Window Layout'
+        class sidebar_on_right(defs.option):
+            """Whether the sidebar will appear on the right."""
+            default = False
+            rtype = types.boolean
+            label = 'Sidebar on right'
+        class vertical_sidebar_split(defs.option):
+            """Whether the main sidebar componens will be split by a vertical separator"""
+            rtype = types.boolean
+            default = False
+            label = 'Vertical sidebar split'
+        class small_toolbar(defs.option):
+            """Whether the toolbar will be displayed with small buttons."""
+            rtype = types.boolean
+            default = False
+            label = 'Small toolbar'
+        class sidebar_width(defs.option):
+            """The width of the sidebar."""
+            default = 200
+            rtype = types.intrange(75, 1800, 25)
+            hidden = True
 
-splash = SplashWindow()
-#splash.show_all()
+    class window_size(defs.optiongroup):
+        """The starting size of the pida window."""
+        __order__ = ['width', 'height', 'save_on_shutdown']
+        label = 'Window Size'
+        class width(defs.option):
+            """The starting width in pixels."""
+            default = 800
+            rtype = types.intrange(0, 2800, 25)
+            label = 'Width'
+        class height(defs.option):
+            """The starting height in pixels."""
+            default = 600
+            rtype = types.intrange(0, 2800, 25)
+            label = 'Height'
+        class save_on_shutdown(defs.option):
+            """Whether the size will be saved on shutdown."""
+            default = True
+            rtype = types.boolean
+            label = 'Save size across sessions'
+
+    class toolbar_and_menubar(defs.optiongroup):
+        """Options relating to the toolbar and main menu bar."""
+        __order__ = ['toolbar_visible', 'menubar_visible', 'sidebar_visible',
+                     'viewpan_visible']
+        label = 'Toolbar and menu bar'
+        class toolbar_visible(defs.option):
+            """Whether the toolbar will start visible."""
+            rtype = types.boolean
+            default = True
+            label = 'Toolbar visible'
+        class menubar_visible(defs.option):
+            """Whether the menubar will start visible."""
+            rtype = types.boolean
+            default = True
+            label = 'Menubar visible'
+        class sidebar_visible(defs.option):
+            """Whether the sidebar will start visible."""
+            rtype = types.boolean
+            default = True
+            hidden = True
+        class viewpan_visible(defs.option):
+            """Whether the view pan will be visible."""
+            rtype = types.boolean
+            default = True
+            label = 'Show View Pane'
+            hidden = True
+
+    def __markup__(self):
+        return 'Window and view'
 
 class WindowManager(service.service):
     """Class to control the main window."""
 
     display_name = 'View'
 
-    class layout(defs.optiongroup):
-        """The layout options."""
-        class sidebar_on_right(defs.option):
-            """Whether the sidebar will appear on the right."""
-            default = False
-            rtype = types.boolean
-        class vertical_sidebar_split(defs.option):
-            """Whether the main sidebar componens will be split by a vertical separator"""
-            rtype = types.boolean
-            default = False
-        class small_toolbar(defs.option):
-            """Whether the toolbar will be displayed with small buttons."""
-            rtype = types.boolean
-            default = False
-        class sidebar_width(defs.option):
-            """The width of the sidebar."""
-            default = 200
-            rtype = types.intrange(75, 1800, 25)
-
-    class toolbar_and_menubar(defs.optiongroup):
-        """Options relating to the toolbar and main menu bar."""
-        class toolbar_visible(defs.option):
-            """Whether the toolbar will start visible."""
-            rtype = types.boolean
-            default = True
-        class menubar_visible(defs.option):
-            """Whether the menubar will start visible."""
-            rtype = types.boolean
-            default = True
-        class sidebar_visible(defs.option):
-            """Whether the sidebar will start visible."""
-            rtype = types.boolean
-            default = True
+    config_definition = WindowConfig
     
-
-    def init(self):
-        self.__splash = splash
-        self.__splash.message('<b>Starting PIDA</b>')
+    def init(self, *args, **kw):
         self.__acels = gtk.AccelGroup()
         self._create_uim()
 
@@ -120,28 +135,82 @@ class WindowManager(service.service):
 
     def reset(self):
         """Display the window."""
-        if self.opt('layout', 'small_toolbar'):
+        self._create_window()
+        self.toolbar.set_style(gtk.TOOLBAR_ICONS)
+        self.opts.__model_notify__()
+        self.action_group.get_action('window+toggle_sidebar').set_active(True)
+        self.action_group.get_action('window+toggle_viewpan').set_active(True)
+
+    toolbar = None
+    menubar = None
+
+    def cb_layout__small_toolbar(self, val):
+        if self.toolbar is None: return
+        if val:
             size = gtk.ICON_SIZE_SMALL_TOOLBAR
         else:
             size = gtk.ICON_SIZE_LARGE_TOOLBAR
-        tbact = self.action_group.get_action('window+toggle_toolbar')
-        tbact.set_active(self.opt('toolbar_and_menubar',
-                                      'toolbar_visible'))
-        menact = self.action_group.get_action('window+toggle_menubar')
-        menact.set_active(self.opt('toolbar_and_menubar',
-                                       'menubar_visible'))
-        sidact = self.action_group.get_action('window+toggle_sidebar')
-        sidact.set_active(self.opt('toolbar_and_menubar',
-                                       'sidebar_visible'))
-        self._show_menubar()
-        self._show_toolbar()
-        self._create_window()
         self.toolbar.set_icon_size(size)
 
+    def cb_toolbar_and_menubar__toolbar_visible(self, val):
+        if self.toolbar is None: return
+        if val:
+            self.toolbar.show_all()
+        else:
+            self.toolbar.hide_all()
+        act = self.action_group.get_action('window+toggle_toolbar')
+        if act.get_active() != val:
+            act.set_active(val)
+
+    def cb_toolbar_and_menubar__menubar_visible(self, val):
+        if self.menubar is None: return
+        if val:
+            self.menubar.show_all()
+        else:
+            self.menubar.hide_all()
+        act = self.action_group.get_action('window+toggle_menubar')
+        if act.get_active() != val:
+            act.set_active(val)
+
+    # not used, left for tiago
+    def _cb_toolbar_and_menubar__sidebar_visible(self, val):
+        if self.menubar is None: return
+        if val:
+            self.show_sidebar()
+        else:
+            self.hide_sidebar()
+        act = self.action_group.get_action('window+toggle_sidebar')
+        if act.get_active() != val:
+            act.set_active(val)
+
+    # not used, left for tiago
+    def _cb_toolbar_and_menubar__viewpan_visible(self, val):
+        if self.menubar is None: return
+        if val:
+            self.show_viewpan()
+        else:
+            self.hide_viewpan()
+        act = self.action_group.get_action('window+toggle_viewpan')
+        if act.get_active() != val:
+            act.set_active(val)
+
+    def reset_gui(self):
+        self.cb_toolbar_and_menubar__toolbar_visible(
+                self.opts.toolbar_and_menubar__toolbar_visible)
+        self.cb_toolbar_and_menubar__menubar_visible(
+                self.opts.toolbar_and_menubar__menubar_visible)
+
+    def stop(self):
+        if self.opts.window_size__save_on_shutdown:
+            w, h = self.__window.get_size()
+            self.opts.window_size__width = w
+            self.opts.window_size__height = h
+
     def cmd_show_window(self):
+        # TODO: try to optimize this, maybe show_all is not optimized
+        # TODO: and explicit show()s make it faster
         self.__window.show_all()
-        self._show_menubar()
-        self._show_toolbar()
+        self.reset_gui()
 
     def cmd_update_action_groups(self):
         self.__uim.ensure_update()
@@ -152,7 +221,13 @@ class WindowManager(service.service):
         ht.add(self.toolbar)
 
     def cmd_append_page(self, bookname, view):
-        self._append_page(bookname, view)
+        self._cm.add(bookname, view)
+
+    def cmd_remove_page(self, view):
+        self._cm.remove(view)
+
+    def cmd_raise_page(self, view):
+        self._cm.raise_view(view)
 
     def cmd_remove_pages(self, bookname):
         if bookname in self.__viewbooks:
@@ -212,14 +287,17 @@ class WindowManager(service.service):
 
     def cmd_previous_view(self):
         """Jump to the previos view in the viewbook."""
-        self.bookview.previous_page()
+        self.bookview.prev_page()
 
     def bnd_buffermanager_document_changed(self, document):
-        self.call('set_title', title=document.filename)
+        if document.is_new:
+            title = 'New File %s' % document.newfile_index
+        else:
+            title = document.filename
+        self.cmd_set_title(title)
 
     def bnd_editormanager_started(self):
-        self.__splash.destroy()
-        self.call('show_window')
+        self.cmd_show_window()
 
     @actions.action(label='Next View',
                     default_accel='<Alt>Up')
@@ -239,9 +317,7 @@ class WindowManager(service.service):
         label='Too_lbar'
         )
     def act_toggle_toolbar(self, action):
-        self.set_option('toolbar_and_menubar', 'toolbar_visible',
-                        action.get_active())
-        self._show_toolbar()
+        self.opts.toolbar_and_menubar__toolbar_visible = action.get_active()
 
     @actions.action(
         type=actions.TYPE_TOGGLE,
@@ -249,9 +325,7 @@ class WindowManager(service.service):
         label='Menubar'
         )
     def act_toggle_menubar(self, action):
-        self.set_option('toolbar_and_menubar', 'menubar_visible',
-                        action.get_active())
-        self._show_menubar()
+        self.opts.toolbar_and_menubar__menubar_visible = action.get_active()
 
     @actions.action(
         type=actions.TYPE_TOGGLE,
@@ -263,48 +337,50 @@ class WindowManager(service.service):
             self.show_sidebar()
         else:
             self.hide_sidebar()
-        self.set_option('toolbar_and_menubar', 'sidebar_visible',
-                        action.get_active())
+        #self.opts.toolbar_and_menubar__sidebar_visible = action.get_active()
 
-    def _show_toolbar(self):
-        if self.opt('toolbar_and_menubar', 'toolbar_visible'):
-            self.toolbar.show_all()
+    @actions.action(
+        type=actions.TYPE_TOGGLE,
+        default_accel='<Control><Shift>v',
+        label='Pane Viewer'
+    )
+    def act_toggle_viewpan(self, action):
+        if action.get_active():
+            self.show_viewpan()
         else:
-            self.toolbar.hide_all()
+            self.hide_viewpan()
+        #self.opts.toolbar_and_menubar__viewpan_visible = action.get_active()
 
-    def _show_menubar(self):
-        if self.opt('toolbar_and_menubar', 'menubar_visible'):
-            self.menubar.show_all()
-        else:
-            self.menubar.hide_all()
-    
+    @actions.action(
+        label='Focus to editor',
+        default_accel='<Shift><Control>e'
+    )
+    def act_focus_editor(self, action):
+        self.boss.call_command('editormanager', 'grab_focus')
+
+    @actions.action(
+        label='Focus to terminal',
+        default_accel='<Shift><Control>y'
+    )
+    def act_focus_terminal(self, action):
+        self.bookview.set_current_page_focused()
 
     def _bind_views(self):
-        self.contentview = contentbook.ContentBook()
-        self.editorview = contentbook.ContentBook(show_tabs=False)
-        self.bookview = contentbook.ContentBook()
-        self.bookview.notebook.set_tab_pos(gtk.POS_TOP)
-        self.externalview = window.external_book()
-        self.pluginview = contentbook.ContentBook()
-        self.__viewbooks = {'content': self.contentview,
-                            'view': self.bookview,
-                            'plugin': self.pluginview,
-                            'edit': self.editorview,
-                            'ext': self.externalview}
-        self.menubar = self.__uim.get_toplevels(gtk.UI_MANAGER_MENUBAR)[0]
-        self.toolbar = self.__uim.get_toplevels(gtk.UI_MANAGER_TOOLBAR)[0]
+        self._cm = ContentManager()
+        self.contentview = self._cm.create_book('content')
+        self.contentview.set_tab_pos(gtk.POS_RIGHT)
+        self.bookview = self._cm.create_book('view', autohide=True)
+        self.editorview = self._cm.create_book('edit')
+        self.editorview.set_show_tabs(False)
+        self.pluginview = self._cm.create_book('plugin')
+        def _s():
+            self.contentview.set_current_page(0)
+        gtk.idle_add(_s)
 
     def _bind_pluginviews(self):
-        for service in self.boss.services:
-            if service.plugin_view_type is not None:
-                if service.NAME in ['buffermanager', 'projectmanager',
-                                    'filemanager']:
-                    self.contentview.append_page(service.plugin_view)
-                else:
-                    self.pluginview.append_page(service.plugin_view)
-        def _s():
-            self.contentview.notebook.set_current_page(0)
-        gtk.idle_add(_s)
+        for service in ['buffermanager', 'filemanager', 'projectmanager']:
+            svc = self.boss.get_service(service)
+            svc.show_view(view=svc.plugin_view)
 
     def _pack(self):
         self.__mainbox = gtk.VBox()
@@ -318,32 +394,31 @@ class WindowManager(service.service):
         self.__mainbox.pack_start(self.__toolbox, expand=False)
 
     def _pack_panes(self):
-        sidebar_on_right = self.opt('layout', 'sidebar_on_right')
-        self.__sidebar = p0 = shiftpaned.ShiftPaned(gtk.HPaned,
+        sidebar_on_right = self.opts.layout__sidebar_on_right
+        self.__sidebar = p0 = shiftpaned.SidebarPaned(gtk.HPaned,
                                                     sidebar_on_right)
         self.__mainbox.pack_start(p0)
-        sidebar_width = self.opt('layout', 'sidebar_width')
+        sidebar_width = self.opts.layout__sidebar_width
         sidebar = self._pack_sidebar()
-        sidebar.show()
         if sidebar_on_right:
             main_pos = 800 - sidebar_width
-            self.contentview.notebook.set_tab_pos(gtk.POS_LEFT)
-            self.pluginview.notebook.set_tab_pos(gtk.POS_LEFT)
+            self.contentview.set_tab_pos(gtk.POS_LEFT)
+            self.pluginview.set_tab_pos(gtk.POS_LEFT)
         else:
             main_pos = sidebar_width
-            self.contentview.notebook.set_tab_pos(gtk.POS_RIGHT)
-            self.pluginview.notebook.set_tab_pos(gtk.POS_RIGHT)
-        p1 = gtk.VPaned()
-        p1.show()
+            self.contentview.set_tab_pos(gtk.POS_RIGHT)
+            self.pluginview.set_tab_pos(gtk.POS_RIGHT)
+        self.__viewpan = p1 = shiftpaned.SidebarPaned(gtk.VPaned, True)
         p0.pack_sub(sidebar, resize=False)
         p0.pack_main(p1, resize=True)
         p0.set_position(main_pos)
-        p1.pack1(self.editorview, resize=True)
-        p1.pack2(self.bookview, resize=False)
-        p1.set_position(430)
+        p1.pack_main(self.editorview, resize=True)
+        p1.pack_sub(self.bookview, resize=False)
+        h = self.opts.window_size__height
+        p1.set_position(h - 350)
 
     def _pack_sidebar(self):
-        sidebar_horiz = self.opt('layout', 'vertical_sidebar_split')
+        sidebar_horiz = self.opts.layout__vertical_sidebar_split
         if sidebar_horiz:
             box = gtk.HPaned()
         else:
@@ -362,12 +437,11 @@ class WindowManager(service.service):
             self.__window.add_accel_group(self.__acels)
             self._connect_drag_events()
             self.__window.add(self.__mainbox)
-            self.__window.resize(800, 600)
-            self.__window.set_icon(im.get_pixbuf())
+            w = self.opts.window_size__width
+            h = self.opts.window_size__height
+            self.__window.resize(w, h)
+            self.__window.set_icon(create_pida_icon())
 
-    def _append_page(self, bookname, page):
-        self.__viewbooks[bookname].append_page(page)
-        self.__viewbooks[bookname].show_all()
 
     def _create_uim(self):
         self.__uim = gtk.UIManager()
@@ -403,7 +477,10 @@ class WindowManager(service.service):
                         <placeholder name="SubPreferencesMenu" />
                         <placeholder name="PreferencesMenu" />
                     </menu>
-                    <menu name="base_project" action="base_project_menu" />
+                    <menu name="base_project" action="base_project_menu">
+                        <placeholder name="ProjectMain" />
+                        <placeholder name="ProjectExtras" />
+                    </menu> 
                     <menu name="base_python" action="base_python_menu" />
                     <menu name="base_tools" action="base_tools_menu">
                         <placeholder name="ToolsMenu" />
@@ -414,6 +491,8 @@ class WindowManager(service.service):
                         <separator />
                     </menu>
                     <menu name="base_view" action="base_view_menu">
+                        <placeholder name="TopViewMenu" />
+                        <separator />
                         <placeholder name="ViewMenu" />
                         <separator />
                     </menu>
@@ -438,6 +517,8 @@ class WindowManager(service.service):
         self.call('register_action_group',
                     actiongroup=ag,
                     uidefinition=menudef)
+        self.menubar = self.__uim.get_toplevels(gtk.UI_MANAGER_MENUBAR)[0]
+        self.toolbar = self.__uim.get_toplevels(gtk.UI_MANAGER_TOOLBAR)[0]
         self.__started = False
 
     def _debug_uim(self):
@@ -488,19 +569,23 @@ class WindowManager(service.service):
                     <menu name="base_view" action="base_view_menu" >
                         <placeholder name="ViewMenu">
                             <separator />
+                            <menuitem name="togglesidebar" action="window+toggle_sidebar" />
+                            <menuitem action="window+toggle_viewpan" />
                             <menuitem name="toggletoolbar" action="window+toggle_toolbar" />
                             <menuitem name="togglemenubar" action="window+toggle_menubar" />
-                            <menuitem name="togglesidebar" action="window+toggle_sidebar" />
                             <separator />
                             <menuitem action="window+next_view" />
                             <menuitem action="window+previous_view" />
+                            <separator />
+                            <menuitem action="window+focus_editor" />
+                            <menuitem action="window+focus_terminal" />
                         </placeholder>
                     </menu>
                 </menubar>
                """
 
     def get_sidebar_on_right(self):
-        return self.opt('layout', 'sidebar_on_right')
+        return self.opts.layout__sidebar_on_right
     
     def show_sidebar(self):
         self.__sidebar.show_sub()
@@ -508,4 +593,10 @@ class WindowManager(service.service):
     def hide_sidebar(self):
         self.__sidebar.hide_sub()
 
+    def show_viewpan(self):
+        self.__viewpan.show_sub()
+
+    def hide_viewpan(self):
+        self.__viewpan.hide_sub()
+
 Service = WindowManager
