@@ -90,16 +90,18 @@ and Vim it didn't start.
 """
 # Gtk imports
 import gtk
-import gobject
 import gtk.gdk as gdk
+import gobject
 # System imports
 import os
 import pty
-import sys
 import time
+import tempfile
 
 class poller(object):
     """
+    DEPRECATED: WE DO NOT USE THIS ANYMORE
+
     An instance of Vim on a pseudoterminal which can be reliably polled.
 
     This class is used to provide an instance of Vim which can be communicated
@@ -228,18 +230,12 @@ class communication_window(gtk.Window):
         # A dictionary to store the working directories for each Vim so they
         # only have to be fetched once.
         self.server_cwds = {}
-        # A list of the last fetched server list, used todecide whther to feed
-        # the most recent server list to the client.
-        self.oldservers = None
         # An instance of the root window, so it only has to be fetched once.
         self.root_window = gdk.get_default_root_window()
-        # Instantiate and start the hidden communication window, used for
-        # fetching accurate and reliable server lists. 
-        self.vim_hidden = poller()
-        self.vim_hidden.start()
-        # Start the timer for fetching the server list at the specified number
-        # of milliseconds (500).
-        gobject.timeout_add(1000, self.fetch_serverlist)
+        # fetch the serverlist to begin with to know when we are started
+        self.oldservers = None
+        self.keep_fetching_serverlist = True
+        gobject.timeout_add(250, self.fetch_serverlist)
     
     def fetch_serverlist(self):
         """
@@ -267,11 +263,9 @@ class communication_window(gtk.Window):
                 self.oldservers = serverlist
                 # A ew serverlist to feed to the client.
                 self.feed_serverlist(serverlist)
-        # Fetch the server list from the hidden vim instance with the
-        # gotservers function as a callback.
-        self.get_hidden_serverlist(gotservers)
-        # Return True so that the timer continues
-        return True
+        gotservers(self.get_rootwindow_serverlist())
+        # decide whether to keep fetching server list
+        return self.keep_fetching_serverlist
 
     def get_rootwindow_serverlist(self):
         """
@@ -312,12 +306,15 @@ class communication_window(gtk.Window):
 
     def get_shell_serverlist(self):
         """
+        DEPRACATED: WE NEVER NEED A SERVERLIST
+        (This is here for educative purposes)
+
         Get the server list by starting console Vim on a Pipe.
 
         This blocks, so we don't use it. It is one of the alternative methods
         of retrieving an accurate serverlist. It is slow, and expensive.
         """
-        vimcom = prop_main_registry.commands.vim_console.value()
+        vimcom = 'gvim'
         p = os.popen('%s --serverlist' % vimcom)
         servers = p.read()
         p.close()
@@ -325,6 +322,9 @@ class communication_window(gtk.Window):
  
     def get_hidden_serverlist(self, callbackfunc):
         """
+        DEPRACATED: WE NEVER NEED A SERVERLIST
+        (This is here for educative purposes)
+
         Get the serverlist from the hidden Vim instance and call the callback
         function with the results.
 
@@ -404,9 +404,6 @@ class communication_window(gtk.Window):
         #self.do_evt('serverlist', serverlist)
         self.cb.vim_new_serverlist(serverlist)
 
-    def init_server(self, servername):
-        self.send_ex(servername, '%s' % VIMSCRIPT)
-
     def fetch_cwd(self, servername):
         """
         Fetch the working directory for a named server and store the result.
@@ -517,6 +514,9 @@ class communication_window(gtk.Window):
     def change_buffer(self, server, filename):
         self.send_ex(server, "exe 'b!'.bufnr('%s')" % filename)
 
+    def change_buffer_number(self, server, number):
+        self.send_ex(server, "b!%s" % number)
+
     def close_buffer(self, server, buffername):
         self.send_ex(server, "exe 'confirm bw'.bufnr('%s')" % buffername)
 
@@ -538,6 +538,11 @@ class communication_window(gtk.Window):
     def open_file(self, server, name):
         self.send_ex(server, 'confirm e %s' % self.escape_filename(name))
 
+    def new_file(self, server):
+        f, path = tempfile.mkstemp()
+        self.open_file(server, path)
+        return path
+
     def goto_line(self, server, linenumber):
         self.send_ex(server, '%s' % linenumber)
         self.send_esc(server)
@@ -547,6 +552,9 @@ class communication_window(gtk.Window):
     def revert(self, server):
         self.send_ex(server, 'e')
 
+    def load_script(self, server, scriptpath):
+        self.send_ex(server, 'so %s' % scriptpath)
+
     def preview_file(self, server, fn):
         self.send_ex(server, 'pc')
         self.send_ex(server, 'set nopreviewwindow')
@@ -575,6 +583,10 @@ class communication_window(gtk.Window):
     def save(self, server):
         self.send_ex(server, 'w')
 
+    def save_as(self, server, filename):
+        print filename
+        self.send_ex(server, 'saveas %s' % filename)
+
     def undo(self, server):
         self.send_esc(server)
         self.send_keys(server, 'u')
@@ -604,8 +616,13 @@ class communication_window(gtk.Window):
         self.send_ex(server, 'set guioptions%s=m' % op)
 
     def quit(self, server):
-        self.send_ex(server, 'q')
+        self.send_ex(server, 'q!')
 
+    def show_sign(self, server, index, filename, line):
+        # vim out by one everywhere!
+        self.send_ex(server, 'sign place %s line=%s name=break file=%s' %
+                             (index + 1, line + 1, filename))
+   
     def cb_notify(self, *a):
         win, ev =  a
         if hasattr(ev, 'atom'):
@@ -657,6 +674,9 @@ let i = 1
     endwhile
     return lis
 endfunction
+:silent function! BreakPoint(l)
+    call Async_event(v:servername.":set_breakpoint,".a:l)
+endfunction
 :silent function! Yank_visual()
     y
     return @"
@@ -669,11 +689,12 @@ endfunction
         echo c
     endtry
 endfunction
+:silent sign define break text=!B
 :silent augroup pida
 :set guioptions-=T
 :set guioptions-=m
 :silent au! pida
-:silent au pida BufEnter * call Async_event(v:servername.":bufferchange,".getcwd().",".bufname('%'))
+:silent au pida BufEnter * call Async_event(v:servername.":bufferchange,".getcwd().",".bufname('%').",".bufnr('%'))
 :silent au pida BufWipeout * call Async_event(v:servername.":bufferunload,".expand('<amatch>'))
 :silent au pida VimLeave * call Async_event(v:servername.":shutdown,")
 :silent au pida VimEnter * call Async_event(v:servername.":started,")
